=head1 NAME

LedgerSMB::DBObject::Budget

=cut

package LedgerSMB::DBObject::Budget;
use strict;
our $VERSION = 0.1;

=head1 SYNOPSIS

This module provides budget management routines, such as entering budgets,
approving or rejecting them, and marking them obsolete.  It does not include
more free-form areas like reporting.  For those, see
LedgerSMB::DBObject::Budget_Report.

=head1 INHERITANCE

=over

=item LedgerSMB

=item LedgerSMB::DBObject

=back

=cut

use base qw(LedgerSMB::DBObject);

=head1 PROPERTIES

=over

=item   $id INT
   The id of the budget

=item   $start_date date
   The start date of the budget, inclusive

=item   $end_date date
   The end date of the budget, inclusive

=item   $reference text
   This is a text reference identifier for the budget

=item   $description text
   This is a text field for the budget description.  It is searchable.

=item   $entered_by int
   Entity id of the employee or contractor who entered the budget

=item   $approved_by int
   Entity id of the employee or contractor who approved the budget

=item   $obsolete_by int
   Entity id for the employee or contractor who marked the budget obsolete

=item   $entered_at timestamp
   Time the budget was entered

=item   $approved_at timestamp
   Time the budget was approved

=item   $obsolete_at timestamp
   Time the budget was deleted

=item   $entered_by_name text
   Name of entity who entered the budget. 

=item   $approved_by_name text
   Name of entity who approved the budget

=item   $obsolete_by_name text
   Name of entity who obsoleted the budget

=item   $department_id int
   ID of department for which the budget is for

=item   $department_name text
   Name of the department for which the budget is for

=item   $project_id int
  ID  of project of which the budget is for

=item  $projectnumber text
   Text descriptor of the project of which the budget is for

=item   @lines 
   These are the actual lines of the budget.  Each one is a hashref containing

=over

=item $budget_id int
   Optional.  Don't use.  Use the $id field of the parent instead.

=item $account_id int
   The id of the chart of accounts entry

=item $accno text
   The account number for the coa entry

=item $amount numeric
   The amount budgetted

=item $description text
   Description of line item

=back

=item @notes
Where each note is a hashref containing

=over 

=item $subject string
   Subject of note

=item $note string
   The body of the note.

=item $created timestamp
   This is when the note was created

=item $created_by string
   Username of the individual who created the note at the time of its creation.

=back

=back

=head1 METHODS

=over

=item save

Saves the current budget.

=cut

sub save {
    my ($self) = @_;
    my ($ref) = $self->exec_method({funcname => 'budget__save_info'});
    $self->merge($ref); # sets defaults
    $self->{details} = [];
    for my $line (@{$self->{lines}}){
       my $l_info = [$line->{account_id}, 
                     $line->{description}, 
                     $line->{amount},
       ];
       push @{$self->{details}}, $l_info;
    }
    $self->exec_method({funcname => 'budget__save_details'});
    $self->{dbh}->commit;
}

=item 

=item search
This method uses the object as the search criteria.  Nulls/undefs match all 
values.  The properties used are:

=over

=item start_date
Matches the start date of the budget.  Full match only.

=item end_date
Matches the end date of the budget.  Full match only

=item includes_date
This date is between start date and end date of budget, inclusive.

=item reference
Partial match on budget reference

=item description
Full text search against description

=item entered_by
Exact match of entered by.

=item approved_by
Exact match of approved by

=item department_id
Exact match of department_id

=item project_id
Exact match of project_id

=item is_approved
true lists approved budgets, false lists unapproved budgets.  null/undef lists 
all.

=item is_obsolete
true lists obsolete budgets. False lists non-obsolete budgets.  null/undef lists
all.

=back

=cut 

sub search {
    my ($self) = @_; # self is search criteria here.
    @{$self->{search_results}}
       = $self->exec_method({funcname => 'budget__search'});
    return @{$self->{search_results}}; 
}

=item get(id)
takes a new (base) object and populates with info for the budget.

=cut

sub get {
   my ($self, $id) = @_;
   my ($info) = $self->call_procedure(
          procname => 'budget__get_info', args => [$id]
   );
   $self->merge($info);
   @{$self->{lines}} = $self->exec_method({funcname => 'budget__get_details'});
   @{$self->{notes}} = $self->exec_method({funcname => 'budget__get_notes'});
}

=item approve
Marks the budget as approved.

=cut

sub approve {
   my ($self) = @_;
   $self->exec_method({funcname => 'budget__approve'});
   $self->{dbh}->commit;
}

=item reject
Reject and deletes the budget.

=cut

sub reject {
   my ($self) = @_;
   $self->exec_method({funcname => 'budget__reject'});
   $self->{dbh}->commit;
}

=item obsolete
Marks the budget as obsolete/superceded.

=cut

sub obsolete {
   my ($self) = @_;
   $self->exec_method({funcname => 'budget__mark_obsolete'});
   $self->{dbh}->commit;
}

=item save_note(subject string, note string)
Attaches a note with this subject and content to the budget.

=cut

sub save_note {
   my ($self, $subject, $note) = @_;
   my ($info) = $self->call_procedure(
          {procname => 'budget__save_note', 
           args => [$self->{id}, $subject, $note]}
   );
}

=item list_projects
Lists all projects available for the start date of the budget.

=cut

sub list_projects {
    my ($self) = @_;
    my $effective_date = $self->{start_date} || 'today';
    return $self->call_procedure(procname => 'project_list_open', 
                                     args =>[$effective_date]);
} 

=item list_departments
Lists all departments

=cut

sub list_departments {
    my ($self) = @_;
    return $self->call_procedure(procname => 'department_list');
}

1;

=back

=head1 SEE ALSO

=over

=item LedgerSMB

=item LedgerSMB::DBObject

=item LedgerSMB::DBObject::Budget_Report

=back

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2011 LedgerSMB Core Team.  This file is licensed under the GNU 
General Public License version 2, or at your option any later version.  Please
see the included License.txt for details.

